# Detect the distribution in use
%global __despace head -n 1 | tr -d '[:space:]' | sed -e 's/[(].*[)]//g'
%global __lower4 cut -c 1-4 | tr '[:upper:]' '[:lower:]'
%global __distfile %([ -f /etc/SuSE-release ] && echo /etc/SuSE-release || echo /etc/redhat-release)
%global __distinit %(sed -e 's/ release .*//' -e 's/\\([A-Za-z]\\)[^ ]*/\\1/g' %{__distfile} | %{__despace} | %{__lower4})
%global __distvers %(sed -e 's/.* release \\([^. ]*\\).*/\\1/' %{__distfile} | %{__despace})
# Identify Alma, CentOS, CentOS Stream and Rocky Linux as rhel
%if "%{__distinit}" == "a" || "%{__distinit}" == "c" || "%{__distinit}" == "cl" || "%{__distinit}" == "cs" || "%{__distinit}" == "rl"
%global __distinit rhel
%endif
# Dist tag for Fedora is still "fc"
%if "%{__distinit}" == "f"
%global __distinit fc
%endif

# RPM and CPAN versioning work differently
%global cpanversion 0.2501
%global rpm_version 0.25.01

# noarch, but to avoid *.list files interfering with signature test
%global debug_package %{nil}

# Store keys in a temp directory
%global gnupghome %(mktemp --directory)

Name:		perl-Exception-Base
Epoch:		1
Version:	%{rpm_version}
Release:	25.%{__distinit}%{__distvers}
Summary:	Lightweight exceptions
License:	GPL-1.0-or-later OR Artistic-1.0-Perl
URL:		https://metacpan.org/release/Exception-Base
Source0:	https://cpan.metacpan.org/modules/by-module/Exception/Exception-Base-%{cpanversion}.tar.gz
Source1:	C4395C9C.pub
Source2:	to_string_changes_errors.t
Patch0:		Exception-Base-0.2501-smartmatch.patch
BuildArch:	noarch
# Module Build
BuildRequires:	coreutils
BuildRequires:	findutils
BuildRequires:	gnupg2
BuildRequires:	make
BuildRequires:	perl-generators
BuildRequires:	perl-interpreter
BuildRequires:	perl(ExtUtils::MakeMaker)
# Module Runtime
BuildRequires:	perl(constant)
BuildRequires:	perl(overload)
BuildRequires:	perl(Scalar::Util)
BuildRequires:	perl(strict)
BuildRequires:	perl(Symbol)
BuildRequires:	perl(warnings)
# Test Suite
BuildRequires:	perl(base)
BuildRequires:	perl(Carp)
BuildRequires:	perl(Cwd)
BuildRequires:	perl(File::Basename)
BuildRequires:	perl(File::Spec)
BuildRequires:	perl(if)
BuildRequires:	perl(Test::Unit::Lite)
# Extra Tests
BuildRequires:	perl(Benchmark)
BuildRequires:	perl(ExtUtils::Manifest)
BuildRequires:	perl(File::Find)
BuildRequires:	perl(File::Slurp)
BuildRequires:	perl(lib)
BuildRequires:	perl(Readonly)
BuildRequires:	perl(Test::Builder)
BuildRequires:	perl(Test::CheckChanges)
BuildRequires:	perl(Test::CPAN::Changes)
BuildRequires:	perl(Test::CPAN::Meta)
BuildRequires:	perl(Test::DistManifest)
BuildRequires:	perl(Test::Distribution)
BuildRequires:	perl(Test::EOL)
BuildRequires:	perl(Test::Kwalitee)
BuildRequires:	perl(Test::MinimumVersion)
BuildRequires:	perl(Test::More) >= 0.88
BuildRequires:	perl(Test::NoTabs)
BuildRequires:	perl(Test::Perl::Critic)
BuildRequires:	perl(Test::Pod) >= 1.14
BuildRequires:	perl(Test::Pod::Coverage) >= 1.04
BuildRequires:	perl(Test::Signature)
BuildRequires:	perl(Test::Spelling), hunspell-en
BuildRequires:	sed
# Test for https://bugzilla.redhat.com/show_bug.cgi?id=1273668
BuildRequires:	perl(Storable)
# Dependencies
%if 0%{?fedora} < 38 && 0%{?rhel} < 10
Requires:	perl(:MODULE_COMPAT_%(eval "`perl -V:version`"; echo $version))
%endif
Requires:	perl(Scalar::Util)
Requires:	perl(Symbol)

%description
This class implements a fully OO exception mechanism similar to
Exception::Class or Class::Throwable. It provides a simple interface
allowing programmers to declare exception classes. These classes can be
thrown and caught. Each uncaught exception prints full stack trace if the
default verbosity is increased for debugging purposes.

%prep
%setup -q -n Exception-Base-%{cpanversion}

# Fix FTBFS with Perl 5.38 onwards (rhbz#2222742)
# Smartmatch is deprecated, resulting warning causes test failures
# Note: if this gets upstreamed, we can start running the signature test again
%patch -P 0

# Drop exec bits from examples to avoid doc-file dependencies
chmod -c -x examples/*

# Import upstream's GPG key so we don't need to fetch it from a keyserver
# when running the signature test
export GNUPGHOME=%{gnupghome}
gpg2 --import %{SOURCE1}

%build
perl Makefile.PL INSTALLDIRS=vendor
make %{?_smp_mflags}

%install
make pure_install DESTDIR=%{buildroot}
find %{buildroot} -type f -name .packlist -delete
%{_fixperms} -c %{buildroot}

%check
export GNUPGHOME=%{gnupghome}

# === MAIN TEST SUITE === #
make test

# ===  RELEASE TESTS  === #
RELEASE_TESTS="$(echo xt/*.t)"

# Don't run the copyright test as it will fail after the year of module release
RELEASE_TESTS="$(echo $RELEASE_TESTS | sed 's|xt/copyright.t||')"

# Don't run the spelling test as we need to add extra stopwords
RELEASE_TESTS="$(echo $RELEASE_TESTS | sed 's|xt/pod_spell.t||')"

# Don't run the critic test as it'll fail regarding builtin keyword catch on recent perls
RELEASE_TESTS="$(echo $RELEASE_TESTS | sed 's|xt/perlcritic.t||')"

# Don't run the signature test because we patched the test suite
RELEASE_TESTS="$(echo $RELEASE_TESTS | sed 's|xt/signature.t||')"

# Signature test would fail on recent distros due to presence of MYMETA.*
[ -f MYMETA.yml ] && mv MYMETA.yml ..
[ -f MYMETA.json ] && mv MYMETA.json ..

make test TEST_FILES="$RELEASE_TESTS"

# Put any MYMETA.* files back where they were
[ -f ../MYMETA.yml ] && mv ../MYMETA.yml .
[ -f ../MYMETA.json ] && mv ../MYMETA.json .

# Fix the POD Spell test and run it
(
	echo "'Unknown"
	echo "'is'"
	echo "'message'"
	echo "'s"
	echo "'value'"
	echo "10x"
	echo "15x"
	echo "exception'"
	echo "informations"
	echo "doesn"
	echo "shouldn"
) >> xt/pod_spellrc
make test TEST_FILES=xt/pod_spell.t

# to_string() appends 'undef' to array attribute
# https://bugzilla.redhat.com/show_bug.cgi?id=1273668
# https://github.com/dex4er/perl-Exception-Base/issues/3
make test TEST_FILES=%{SOURCE2}

%clean
rm -rf %{buildroot} %{gnupghome}

%files
%license LICENSE
%doc Changes Incompatibilities README examples/
%{perl_vendorlib}/Exception/
%{_mandir}/man3/Exception::Base.3*

%changelog
* Fri May  3 2024 Paul Howarth <paul@city-fan.org> - 1:0.25.01-25
- Avoid use of %%{rpmversion} macro, now an rpm built-in

* Sun Jul 16 2023 Paul Howarth <paul@city-fan.org> - 1:0.25.01-24
- Fix FTBFS with Perl 5.38 (rhbz#2222742)

* Fri Apr  7 2023 Paul Howarth <paul@city-fan.org> - 1:0.25.01-13
- Use SPDX-format license tag

* Mon May 24 2021 Paul Howarth <paul@city-fan.org> - 1:0.25.01-12
- Avoid perl critic test, which fails on recent perls with built-in catch

* Thu Jun 13 2019 Paul Howarth <paul@city-fan.org> - 1:0.25.01-11
- Modernize spec
  - Use mktemp to create GNUPGHOME, simplifying build process
  - Switch upstream from search.cpan.org to metacpan.org
  - BR: perl-generators unconditionally
  - BR: perl-interpreter rather than perl
  - Don't explicitly clean buildroot in %%install section
  - Drop support for building with Test::More < 0.88
  - Drop legacy BuildRoot: and Group: tags
  - Spell check back-end is always hunspell

* Mon Aug 22 2016 Paul Howarth <paul@city-fan.org> - 1:0.25.01-2
- Fix FTBFS when perl is not in the minimal buildroot
- BR: perl-generators where available
- Simplify find command using -delete
- Add test case to verify fix for #1273668

* Sun Nov 22 2015 Paul Howarth <paul@city-fan.org> - 1:0.25.01-1
- Update to 0.2501
  - Fixes "Redundant argument in sprintf" warning on Perl 5.21.2
- Separate RPM and CPAN versioning
- Update upstream GPG key

* Tue Jun 30 2015 Paul Howarth <paul@city-fan.org> - 1:0.25-4
- Fixed test suite failure for Perl 5.21.2 and higher
- Classify buildreqs by usage

* Wed Sep 17 2014 Paul Howarth <paul@city-fan.org> - 1:0.25-3
- BR: perl(Test::CPAN::Changes) unconditionally
- Drop %%defattr, redundant since rpm 4.4
- Use %%license where possible

* Thu Jul 25 2013 Paul Howarth <paul@city-fan.org> - 1:0.25-2
- Perl 5.18 rebuild

* Sun May 26 2013 Paul Howarth <paul@city-fan.org> - 1:0.25-1
- Update to 0.25
  - Builds on Perl 5.18.0 with smartmatch operator as an experimental feature
- Bump epoch due to upstream reverting to 2-digit version number
- Add buildreqs for new release tests

* Sat Jul 14 2012 Paul Howarth <paul@city-fan.org> - 0.2401-6
- Work around failure of signature test due to MYMETA.* files
- BR: perl(base),perl(constant), perl(Cwd) and perl(File::Spec)
- Don't need to remove empty directories from the buildroot
- Drop support for old distributions prior to FC-3:
  - Perl::Critic is always available
  - Don't need to define %%{perl_vendorlib}
  - Use DESTDIR rather than PERL_INSTALL_ROOT
  - Spelling dictionary is never aspell-en-gb

* Fri Jun 24 2011 Paul Howarth <paul@city-fan.org> - 0.2401-4
- Rebuild for perl 5.14.1 in Rawhide
- Fix dist tag for CentOS 6 and Scientific Linux

* Fri May  6 2011 Paul Howarth <paul@city-fan.org> - 0.2401-3
- Import from Fedora

* Tue Feb 08 2011 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 0.2401-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_15_Mass_Rebuild

* Sun Oct 03 2010 Emmanuel Seyman <emmanuel.seyman@club-internet.fr> - 0.2401-1
- Update to 0.2401

* Sat May 01 2010 Marcela Maslanova <mmaslano@redhat.com> - 0.24-3
- Mass rebuild with perl-5.12.0

* Fri Apr 30 2010 Marcela Maslanova <mmaslano@redhat.com> - 0.24-2
- Mass rebuild with perl-5.12.0

* Sun Apr  4 2010 Emmanuel Seyman <emmanuel.seyman@club-internet.fr> - 0.24-1
- Update to 0.24
- Add perl(Test::Unit::Lite) to BuildRequires

* Mon Dec  7 2009 Stepan Kasal <skasal@redhat.com> - 0.21-4
- Rebuild against perl 5.10.1

* Sat Jul 25 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 0.21-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_12_Mass_Rebuild

* Thu Feb 26 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 0.21-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_11_Mass_Rebuild

* Thu Jan 29 2009 Allisson Azevedo <allisson@gmail.com> - 0.21-1
- Initial RPM release
